/**
 * Allows printing in true black/write regardless of color scheme used.
 *
 * @category  WeBuilder Plugin
 * @package   BW Print
 * @author    Peter Klein <pmk@io.dk>
 * @copyright 2017
 * @license   http://www.freebsd.org/copyright/license.html  BSD License
 * @version   1.0
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *     46   function OnWebkitCreated(sender, browser)
 *     57   function HideWindow(Sender)
 *     83   function OnWebkitLoadEnd(sender, browser, frame, status, res)
 *     95   function OnWebkitData(channel, data)
 *    106   function CreateWebkit()
 *    142   function Print()
 *    153   function PrintPreview()
 *    163   function OnExit()
 *    176   function OnInstalled()
 *
 * TOTAL FUNCTIONS: 9
 * (This index is automatically created/updated by the WeBuilder plugin "DocBlock Comments")
 *
 */

/**
 * Global variables
 */
var win = null;
var web = null;
var editorData = "";
var mode = "print";

/**
 * Callback function: Activated when creation of Webkit is done
 *
 * @param  object   sender
 * @param  ??       browser
 *
 * @return void
 */
function OnWebkitCreated(sender, browser) {
	Script.TimeOut(1, "HideWindow");
}

/**
 * Callback function: Hides Webkit window
 *
 * @param  TScriptableTimer   sender
 *
 * @return void
 */
function HideWindow(Sender) {
	if (mode == "print") {
		win.Caption = "";
		win.Hide();
	}
	else {
		win.Caption = "Print Preview";
		win.width = Script.ReadSetting("Preview Width", "800");
		win.Height = Script.ReadSetting("Preview Height", "600");
	    web.Webkit.Width = win.ClientWidth;
	    web.Webkit.Height = win.ClientHeight;
		win.Show();
	}
}

/**
 * Callback function: Sends editor content to Webkit
 *
 * @param  TChromium  sender
 * @param  integer    browser ?
 * @param  object     frame ?
 * @param  integer    status ?
 * @param  integer    res ?
 *
 * @return void
 */
function OnWebkitLoadEnd(sender, browser, frame, status, res) {
    web.Send(mode, editorData);
}

/**
 * Recieves communication data from webkit html page
 *
 * @param  string   channel
 * @param  mixed   data
 *
 * @return string
 */
function OnWebkitData(channel, data) {
	//Alert(channel + " + " + data);
	win.Hide();
    return;
}

/**
 * Create ScriptableChromium window
 *
 * @return void
 */
function CreateWebkit() {
  if (win == null) {
     win = new TForm(WeBuilder);
     win.width = 60;
     win.Height = 60;
     win.Caption = "Loading...";

     web = Script.CreateScriptableWebKit(win, Script.GetPath + "index.html", "OnWebkitCreated");
     web.OnLoadEnd = "OnWebkitLoadEnd";
     web.Subscribe("print", "OnWebkitData"); // Communication channel
     web.Subscribe("preview", "OnWebkitData"); // Communication channel
     web.Webkit.Top = 0;
     web.Webkit.Left = 0;
     web.Webkit.Width = win.ClientWidth;
     web.Webkit.Height = win.ClientHeight;
     web.Webkit.Anchors = akLeft || akRight || akTop || akBottom;
     win.Show;
  }
    else {
        web.Send(mode, editorData);

		if (mode == "preview") {
			win.Caption = "Print Preview";
			win.width = Script.ReadSetting("Preview Width", "800");
			win.Height = Script.ReadSetting("Preview Height", "600");
		    web.Webkit.Width = win.ClientWidth;
		    web.Webkit.Height = win.ClientHeight;
			win.Show();
		}
}}

/**
 * Print editor content
 *
 * @return void
 */
function Print() {
    editorData = Editor.Text;
	mode = "print";
	CreateWebkit();
}

/**
 * Print Preview editor content
 *
 * @return void
 */
function PrintPreview() {
    editorData = Editor.Text;
	mode = "preview";
	CreateWebkit();
}
/**
 * Cleanup function when plugin is disabled or editor is closed
 *
 * @return void
 */
function OnExit() {
  if (web != null) {
    delete web;
  }
  if (win != null) {
    delete win;
}}

/**
 * Show info when plugin is installed
 *
 * @return void
 */
function OnInstalled() {
  alert("BW Print 1.0 by Peter Klein installed sucessfully!");
}

Script.ConnectSignal("installed", "OnInstalled");
Script.ConnectSignal("exit", "OnExit");

var bmp = new TBitmap;
var icon = LoadFileToBitmap(Script.Path + "print-icon.png", bmp);
var act1 = Script.RegisterDocumentAction("BW Print", "Print", "", "Print");
Actions.SetIcon(act1, bmp);
var act2 = Script.RegisterDocumentAction("BW Print", "Print Preview", "", "PrintPreview");
Actions.SetIcon(act2, bmp);
delete bmp;